/*******************************************************************************
 * Author : Jérôme Dubois       Jerome.Dubois@ensimag.imag.fr jerome@stud.ntnu.no
 * Created: 2008-01-12
 * File  : 1d_filter.c
 * Copyright 2007 Jerome Dubois
 *
 * Goal: Compute a 1D filter. Signals are 1D signals.
         When we refer to 2D, it is a 2D representation of 1D signals.
 ******************************************************************************/

//#include <string.h> // strlen, strncpy
#include <stdlib.h> // exit, EXIT_SUCCESS
#include <stdio.h> // printf, FILE. Already included in grim.h
// #include <stdint.h>
// #include <math.h> // sin, sqrt, pow. Needs linking with option -lm
 #include <assert.h> // assert
//#include <getopt.h> // getopt
#include "filters.h"
#include "dprintf.h"

#define IMAGES "../images/"   // path to the images (to read and create)
#define SAVE(sig,file) save_signal((sig),(file)) 

/*############################################################################*/
/*#                      functions                                           #*/
/*############################################################################*/
grim gtmp;
void save_signal(fgrim *signal, char *file_name){
  //fprintf(stdout,file_name); //fgrim_print(signal);
  fgrim_line_to_2d(signal, 0, &gtmp);
//  strncpy( to, from, count );
  grim_write_bmp(&gtmp, file_name);
  grim_free(&gtmp);
}

/*############################################################################*/
/*#                      main                                                #*/
/*############################################################################*/
int main(int argc, char * argv[]){
  int i;
  int w, w2; // width of the signals. w2 = w/2
  fgrim in_1d, out_1d;
  grim  in_2d, out_2d;
  fgrim i1, i2, i3, i4, i5;
  double val;
  fgrim h0_coef, h1_coef; // low/high pass filter coefficients: h0, h1

  /* prepare input signal (in_1d) */  
  w = 600; 
  assert(0 == w%2);
  w2 = w/2;
  fgrim_create(&i1, w, 1); fgrim_set_sinus(&i1, WHITE, 0, 3);
SAVE(&i1, IMAGES"i1.bmp");
  fgrim_create(&i2, w, 1); fgrim_set_sinus(&i2, WHITE, 0, 20);
SAVE(&i2, IMAGES"i2.bmp");
  fgrim_create(&i3, w, 1); fgrim_set_sinus(&i3, WHITE, 0, 60);
SAVE(&i3, IMAGES"i3.bmp");
  fgrim_create(&i4, w, 1); fgrim_set_sinus(&i4, WHITE, 0, 80);
SAVE(&i4, IMAGES"i4.bmp");
  fgrim_create(&i5, w, 1); fgrim_set_sinus(&i5, WHITE, 0, 100);
SAVE(&i5, IMAGES"i5.bmp");
  
  fgrim_create(&in_1d, w, 1);
  for (i=0; i<w; i++){
    val = 0.5 * i1.pix[i] + 0 * i2.pix[i] + 1 * i3.pix[i] + 0 * i4.pix[i] + 1.5 * i5.pix[i] ;
    in_1d.pix[i] = (float) val / 3.0 ;
  }
  
  fgrim_free(&i1);
  fgrim_free(&i2);
  fgrim_free(&i3);
  fgrim_free(&i4);
  fgrim_free(&i5);
  
  /* signal processing (in_1d -> out_1d) */
    fgrim_create(&out_1d, w, 1);
    
    // coefficient preparation
    init_coef(&h0_coef);
    pair_filter_coef(&h0_coef, &h1_coef);

    // bank preparation
    fgrim *sE1, *sE6;
    fgrim sE2b1, sE3b1, sE4b1, sE5b1;
    fgrim sE2b2, sE3b2, sE4b2, sE5b2;
    
    sE1 = &in_1d;  // pointer copy
    //fgrim_create(&sE2b1, w , 1); fgrim_create(&sE2b2, w , 1);
    fgrim_create(&sE3b1, w2, 1); fgrim_create(&sE3b2, w2, 1);
    fgrim_create(&sE4b1, w , 1); fgrim_create(&sE4b2, w , 1);
    //fgrim_create(&sE5b1, w , 1); fgrim_create(&sE5b2, w , 1);
    sE6 = &out_1d; // pointer copy
    
SAVE(sE1, IMAGES"sE1.bmp");
    // Filters H0(z) and H1(z)
    fgrim_copy(sE1, &sE2b1);
    fgrim_copy(sE1, &sE2b2);
    filter_periodic_horizontal(&sE2b1, &h0_coef, 0);
    filter_periodic_horizontal(&sE2b2, &h1_coef, 0);
SAVE(&sE2b1, IMAGES"sE2b1.bmp"); SAVE(&sE2b2, IMAGES"sE2b2.bmp");
#ifdef NODOWNUP
    /* Here, we don't go through a downsample and upsample, and we copy
       directly the output of the analysis filters to the input of the synthesis
       filters. This gives near-perfect reconstruction.
    */
    fgrim_free(&sE4b1); fgrim_free(&sE4b2);
    fgrim_copy(&sE2b1, &sE4b1);
    fgrim_copy(&sE2b2, &sE4b2);
    printf("Warning: 1d_filter has been compiled with macro NODOWNUP, thus skipping the downsampling and upsampling operations.\n");
#else // #ifdef NODOWNUP
    // Downsample
    for (i=0; i<w2; i++){
      sE3b1.pix[i] = sE2b1.pix[2*i+0];
      sE3b2.pix[i] = sE2b2.pix[2*i+0];
    }
SAVE(&sE3b1, IMAGES"sE3b1.bmp"); SAVE(&sE3b2, IMAGES"sE3b2.bmp");
    // Upsample
    for (i=0; i<w2; i++){
      sE4b1.pix[2*i+0] = sE3b1.pix[i];
      sE4b1.pix[2*i+1] = 0;
      sE4b2.pix[2*i+0] = sE3b2.pix[i];
      sE4b2.pix[2*i+1] = 0;
    }
#endif // #ifdef NODOWNUP
SAVE(&sE4b1, IMAGES"sE4b1.bmp"); SAVE(&sE4b2, IMAGES"sE4b2.bmp");
    // Filters G0(z) and G1(z)
    fgrim_copy(&sE4b1, &sE5b1);
    fgrim_copy(&sE4b2, &sE5b2);
    filter_periodic_horizontal(&sE5b1, &h0_coef, h0_coef.w-1);
    filter_periodic_horizontal(&sE5b2, &h1_coef, h1_coef.w-1);
#ifdef NODOWNUP
    amplify(&sE5b2, -1);
#else // #ifdef NODOWNUP
    amplify(&sE5b1,  2);
    amplify(&sE5b2, -2);
#endif // #ifdef NODOWNUP
SAVE(&sE5b1, IMAGES"sE5b1.bmp"); SAVE(&sE5b2, IMAGES"sE5b2.bmp");

    // Reassemble bands 1 and 2
    for (i=0; i<w; i++){
      sE6->pix[i] = sE5b1.pix[i] + sE5b2.pix[i];
    }
SAVE(sE6, IMAGES"sE6.bmp");
SAVE(sE6, IMAGES"sE0.bmp");
    
    // Free
    sE1 = NULL;
    fgrim_free(&sE2b1);            fgrim_free(&sE2b2);
    fgrim_free(&sE3b1);            fgrim_free(&sE3b2);
    fgrim_free(&sE4b1);            fgrim_free(&sE4b2);
    fgrim_free(&sE5b1);            fgrim_free(&sE5b2);
    sE6 = NULL;
    // Free coeff
    free_coef(&h0_coef);  fgrim_free(&h1_coef);

  /* output */
  fgrim_line_to_2d(&in_1d, 0, &in_2d);
  grim_write_bmp(&in_2d, IMAGES"in_2d.bmp");
  fgrim_line_to_2d(&out_1d, 0, &out_2d);
  grim_write_bmp(&out_2d, IMAGES"out_2d.bmp");
//  printf("input==output: %s\n", fgrim_cmp(&in_1d,&out_1d) ? "yes" : "no";
  printf("difference: %f percent\n", fgrim_diff(&in_1d,&out_1d));

  /* free */
  fgrim_free(&in_1d);
  fgrim_free(&out_1d);
  grim_free(&in_2d);
  grim_free(&out_2d);

  return EXIT_SUCCESS;
}

