/*******************************************************************************
 * Author : Jérôme Dubois       Jerome.Dubois@ensimag.imag.fr jerome@stud.ntnu.no
 * Created: 2008-01-12
 * File  : 1d_polyphase_filter.c
 * Copyright 2007 Jerome Dubois
 *
 * Goal: Compute a 1D polyphase filter. Signals are 1D signals.
         When we refer to 2D, it is a 2D representation of 1D signals.
 ******************************************************************************/

//#include <string.h> // strlen, strncpy
#include <stdlib.h> // exit, EXIT_SUCCESS
#include <stdio.h> // printf, FILE. Already included in grim.h
// #include <stdint.h>
// #include <math.h> // sin, sqrt, pow. Needs linking with option -lm
 #include <assert.h> // assert
//#include <getopt.h> // getopt
#include "filters.h"
#include "dprintf.h"

#define IMAGES "../images/"   // path to the images (to read and create)
#define SAVE(sig,file) save_signal((sig),(file)) 

/*############################################################################*/
/*#                      functions                                           #*/
/*############################################################################*/
grim gtmp;
void save_signal(fgrim *signal, char *file_name){
  //fprintf(stdout,file_name); fgrim_print(signal);
  fgrim_line_to_2d(signal, 0, &gtmp);
//  strncpy( to, from, count );
  grim_write_bmp(&gtmp, file_name);
  grim_free(&gtmp);
}

/*############################################################################*/
/*#                      main                                                #*/
/*############################################################################*/
int main(int argc, char * argv[]){
  int i;
  int w, w2; // width of the signals
  fgrim in_1d, out_1d;
  grim  in_2d, out_2d;
  fgrim i1, i2, i3, i4, i5;
  double val;
  fgrim h0_coef;//, h1_coef; // low/high pass filter coefficients: h0, h1
  fgrim p0_coef, p1_coef;   // polyphase filter coefficients

  /* prepare input signal (in_1d) */  
  w = 600; 
  assert(0 == w%2);
  w2 = w/2;
  fgrim_create(&i1, w, 1); fgrim_set_sinus(&i1, WHITE, 0, 3);
SAVE(&i1, IMAGES"i1.bmp");
  fgrim_create(&i2, w, 1); fgrim_set_sinus(&i2, WHITE, 0, 40);
SAVE(&i2, IMAGES"i2.bmp");
  fgrim_create(&i3, w, 1); fgrim_set_sinus(&i3, WHITE, 0, 60);
SAVE(&i3, IMAGES"i3.bmp");
  fgrim_create(&i4, w, 1); fgrim_set_sinus(&i4, WHITE, 0, 80);
SAVE(&i4, IMAGES"i4.bmp");
  fgrim_create(&i5, w, 1); fgrim_set_sinus(&i5, WHITE, 0, 100);
SAVE(&i5, IMAGES"i5.bmp");
  
  fgrim_create(&in_1d, w, 1);
  for (i=0; i<w; i++){
    val = 0.5 * i1.pix[i] + 0 * i2.pix[i] + 1 * i3.pix[i] + 0 * i4.pix[i] + 1.5 * i5.pix[i] ;
    //val = 1 * i1.pix[i] + 0.5 * i2.pix[i] + 0 * i3.pix[i] + 0 * i4.pix[i] + 1.5 * i5.pix[i] ;
    in_1d.pix[i] = (float) val / 3.0 ;
  }
  
  fgrim_free(&i1);
  fgrim_free(&i2);
  fgrim_free(&i3);
  fgrim_free(&i4);
  fgrim_free(&i5);
  
  /* signal processing (in_1d -> out_1d) */
    fgrim_create(&out_1d, w, 1);
    
    // coefficient preparation
    init_coef(&h0_coef);
    //pair_filter_coef(&h0_coef, &h1_coef); // check fgrim_free(&h1_coef);
    fgrim_create(&p0_coef, (h0_coef.w)/2, 1);
    fgrim_create(&p1_coef, (h0_coef.w)/2, 1);
    for (i=0; i<(h0_coef.w)/2; i++){
      p0_coef.pix[i] = h0_coef.pix[2*i+0];
      p1_coef.pix[i] = h0_coef.pix[2*i+1];
    }

    // bank preparation
    fgrim *sC2, *sC9;
    fgrim sC3a4b1, sC5b1, sC6a7b1; // sC3a4b1: signal C3 and C4 band 1
    fgrim sC3a4b2, sC5b2, sC6a7b2; // sC5b2  : signal C5 band 2
    
    sC2 = &in_1d;  // pointer copy
    fgrim_create(&sC3a4b1, w2, 1); fgrim_create(&sC3a4b2, w2, 1);
    fgrim_create(&sC5b1  , w2, 1); fgrim_create(&sC5b2  , w2, 1);
    fgrim_create(&sC6a7b1, w2, 1); fgrim_create(&sC6a7b2, w2, 1);
    sC9 = &out_1d; // pointer copy
    
SAVE(sC2, IMAGES"sC2.bmp");
    // Downsample
    for (i=0; i<w2; i++){
      sC3a4b1.pix[i] = sC2->pix[2*i  ]; // even samples
      sC3a4b2.pix[i] = sC2->pix[2*i+1]; // odd  samples // Delay of 1: z^{-1}
    }
SAVE(&sC3a4b1, IMAGES"sC3b1.bmp"); SAVE(&sC3a4b2, IMAGES"sC3b2.bmp");
    // Filters P0(z) and P1(z)
    filter_periodic_horizontal(&sC3a4b1, &p0_coef, 0);
    filter_periodic_horizontal(&sC3a4b2, &p1_coef, 0);
SAVE(&sC3a4b1, IMAGES"sC4b1.bmp"); SAVE(&sC3a4b2, IMAGES"sC4b2.bmp");
    // Polyphase first combine
    for (i=0; i<w2; i++){
      sC5b1.pix[i] = sC3a4b1.pix[i] + sC3a4b2.pix[i];
      sC5b2.pix[i] = sC3a4b1.pix[i] - sC3a4b2.pix[i];
    }
SAVE(&sC5b1, IMAGES"sC5b1.bmp");   SAVE(&sC5b2, IMAGES"sC5b2.bmp");
    // Middle treatment
    //for (i=0; i<w2; i++){ sC5b2.pix[i] = 0; } // erase one band
    // Polyphase second combine
    for (i=0; i<w2; i++){
      sC6a7b1.pix[i] = 2*( sC5b1.pix[i] + sC5b2.pix[i] );
      sC6a7b2.pix[i] = 2*( sC5b1.pix[i] - sC5b2.pix[i] );
    }
    // Amplify by 2 // Could be done later, because we are now in range [0,511]
SAVE(&sC6a7b1, IMAGES"sC6b1.bmp"); SAVE(&sC6a7b2, IMAGES"sC6b2.bmp");
    // Filters Q0(z) and Q1(z)
    filter_periodic_horizontal(&sC6a7b1, &p1_coef, p1_coef.w-1);
    filter_periodic_horizontal(&sC6a7b2, &p0_coef, p0_coef.w-1);
SAVE(&sC6a7b1, IMAGES"sC7b1.bmp"); SAVE(&sC6a7b2, IMAGES"sC7b2.bmp");
    // Upsample
    for (i=0; i<w2; i++){
      sC9->pix[2*i  ] = sC6a7b1.pix[i];
      sC9->pix[2*i+1] = sC6a7b2.pix[i];
    }
SAVE(sC9, IMAGES"sC9.bmp");
SAVE(sC9, IMAGES"sC0.bmp");
    
    // Free
    sC2 = NULL;
    fgrim_free(&sC3a4b1);          fgrim_free(&sC3a4b2);
    fgrim_free(&sC5b1);            fgrim_free(&sC5b2);
    fgrim_free(&sC6a7b1);          fgrim_free(&sC6a7b2);
    sC9 = NULL;
    // Free coeff
    free_coef(&h0_coef);  //fgrim_free(&h1_coef);
    fgrim_free(&p0_coef); fgrim_free(&p1_coef);

  /* output */
  fgrim_line_to_2d(&in_1d, 0, &in_2d);
  grim_write_bmp(&in_2d, IMAGES"in_2d.bmp");
  fgrim_line_to_2d(&out_1d, 0, &out_2d);
  grim_write_bmp(&out_2d, IMAGES"out_2d.bmp");
//  printf("input==output: %s\n", fgrim_cmp(&in_1d,&out_1d) ? "yes" : "no";
  printf("difference: %f percent\n", fgrim_diff(&in_1d,&out_1d));

  /* free */
  fgrim_free(&in_1d);
  fgrim_free(&out_1d);
  grim_free(&in_2d);
  grim_free(&out_2d);

  return EXIT_SUCCESS;
}

