/*******************************************************************************
 * Author: Jérôme Dubois       Jerome.Dubois@ensimag.imag.fr jerome@stud.ntnu.no
 * File  : dfb_cuda_gold.cpp
 * Copyright 2008 Jerome Dubois
 *
 * The program uses some code from the NVIDIA CUDA SDK 1.1 sample
 * convolutionSeparable. The license authorizes us to use it and deny all
 * liability. The NVIDIA copyright and license is at the end of the file.
 *
 * The two functions of this file have been used for testing, but are not used
 * anymore.
 ******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
// Reference row convolution filter
////////////////////////////////////////////////////////////////////////////////
extern "C" void convolutionRowCPU(
    float *h_Result,
    float *h_Data,
    float *h_Kernel,
    int dataW,
    int dataH,
    int kernelR
){
    int x, y, k, d;
    float sum;

    for(y = 0; y < dataH; y++)
        for(x = 0; x < dataW; x++){
            sum = 0;
            for(k = 0; k <= kernelR; k++){ // k <= 2*kernelR
                d = x + k + 0;
                if(d < 0 || d >= dataW)
                    d = (d + dataW) % dataW ; // periodic raws
                sum += h_Data[y * dataW + d] * h_Kernel[k];
            }
            h_Result[y * dataW + x] = sum;
        }
}


////////////////////////////////////////////////////////////////////////////////
// Reference column convolution filter
////////////////////////////////////////////////////////////////////////////////
extern "C" void convolutionColumnCPU(
    float *h_Result,
    float *h_Data,
    float *h_Kernel,
    int dataW,
    int dataH,
    int kernelR
){
    int x, y, k, d;
    float sum;

    for(y = 0; y < dataH; y++){
        for(x = 0; x < dataW; x++){
            sum = 0;
            for(k = 0; k <= kernelR; k++){ // k <= 2*kernelR
                d = y + k + 0;
                if(d < 0 || d >= dataH)
                    d = (d + dataH) % dataH ; // periodic column
                sum += h_Data[d * dataW + x] * h_Kernel[k];
            }
            h_Result[y * dataW + x] = sum;
        }
    }
}

/******************************************************************************/
/* 
 * Some code used here is subject to NVIDIA ownership, so we must copy the
 * following disclaimer
 */
 
/*
 * Copyright 1993-2007 NVIDIA Corporation.  All rights reserved.
 *
 * NOTICE TO USER:   
 *
 * This source code is subject to NVIDIA ownership rights under U.S. and 
 * international Copyright laws.  Users and possessors of this source code 
 * are hereby granted a nonexclusive, royalty-free license to use this code 
 * in individual and commercial software.
 *
 * NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE 
 * CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR 
 * IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH 
 * REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF 
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL, 
 * OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS 
 * OF USE, DATA OR PROFITS,  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE 
 * OR OTHER TORTIOUS ACTION,  ARISING OUT OF OR IN CONNECTION WITH THE USE 
 * OR PERFORMANCE OF THIS SOURCE CODE.  
 *
 * U.S. Government End Users.   This source code is a "commercial item" as 
 * that term is defined at  48 C.F.R. 2.101 (OCT 1995), consisting  of 
 * "commercial computer  software"  and "commercial computer software 
 * documentation" as such terms are  used in 48 C.F.R. 12.212 (SEPT 1995) 
 * and is provided to the U.S. Government only as a commercial end item.  
 * Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through 
 * 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the 
 * source code with only those rights set forth herein. 
 *
 * Any use of this source code in individual and commercial software must 
 * include, in the user documentation and internal comments to the code,
 * the above Disclaimer and U.S. Government End Users Notice.
 */
