
#include "grim.h"
#include "filters.h"

/******************************************************************************
 *                       test1
 * Goal:
 *   test of filter_periodic_horizontal(...)
 ******************************************************************************/
void test1(){
    fgrim fg1234;
    float coef[4];
    fgrim gcoef;
    fprintf(stdout,"test1: filter_periodic_horizontal\n");

    // init
    fgrim_create(&fg1234, 4, 1);
    fg1234.pix[0]=1; fg1234.pix[1]=2; fg1234.pix[2]=3; fg1234.pix[3]=4;
    //fg1234.pix[4]=5; fg1234.pix[5]=6; fg1234.pix[6]=7; fg1234.pix[7]=8;
    coef[0]=1; coef[1]=2; coef[2]=2; coef[3]=1;
    gcoef.w = 4; gcoef.h = 1; gcoef.pix = coef;

    // filtering
    filter_periodic_horizontal(&fg1234, &gcoef, 0);

    //check
    fprintf(stdout, "(1,2,3,4)**(1,2,2,1) :\n");
    fgrim_print(&fg1234);
    if ( fg1234.pix[0] != 15 || // 3*1 + 4*2 + 1*2 + 2*1 = 15
         fg1234.pix[1] != 13 || // 4*1 + 1*2 + 2*2 + 3*1 = 13
         fg1234.pix[2] != 15 || // 1*1 + 2*2 + 3*2 + 4*1 = 15
         fg1234.pix[3] != 17    // 2*1 + 3*2 + 4*2 + 1*1 = 17
        )
        fprintf(stderr, "test1: error\n");


}

/******************************************************************************
 *                       test2
 * Goal:
 *   test of filter_periodic_vertical(...)
 ******************************************************************************/
void test2(){
  fgrim filter_coef;
  init_coef(&filter_coef);
  fprintf(stdout,"test2: filter_periodic_vertical NOT IMPLEMENTED\n");
  free_coef(&filter_coef);
}


/******************************************************************************
 *                       test3
 * Goal:
 *   test of {up,down}sample_quincunx_q2(...)
 ******************************************************************************/
#define TEST3_SAVE(sig,file) fgrim_write_bmp((sig),(file)); //printf((file)); printf(": \n"); fgrim_print((sig));
void test3(){
  fgrim in, b1, b2, out; // in out
  int col, raw, first_of_raw;
  int s = 200; // matrix size
  fprintf(stdout,"test3: {up,down}sample_quincunx_q2\n");
  fgrim_create(&in, s, s);
  // We initiate in with something
  for (raw=0; raw<in.h; raw++){
     first_of_raw = (s-1-raw)*s +1;
     // I initate in with that to compare with my hand computed result
     for (col=0; col<in.w; col++){
        in.pix[col + raw*in.w] = first_of_raw + col;
     }
  }
  fgrim_free(&in);
  grim grimin;
  grim_read_grim("../images/twodir.grim",&grimin);
  grim_to_fgrim(&grimin, &in);
  grim_free(&grimin);
  
  // Downsample
  downsample_quincunx_q2(&in, &b1, &b2);
  TEST3_SAVE(&in,"in.bmp");
  TEST3_SAVE(&b1,"b1.bmp");
  TEST3_SAVE(&b2,"b2.bmp");
  
  // Upsample
  upsample_quincunx_q2(&out, &b1, &b2);
  printf("diff=%f percent\n", fgrim_diff(&in,&out) );
  TEST3_SAVE(&out,"outb1b2.bmp");
  
  
  fgrim_free(&in);
  fgrim_free(&b1);
  fgrim_free(&b2);
  fgrim_free(&out);
  return;
}

/******************************************************************************
 *                       test4
 * Goal:
 *   test of xxx(...)
 ******************************************************************************/
void test4(){
  fprintf(stdout,"test4: NOT IMPLEMENTED\n");
  return;
}
/******************************************************************************
 *                       test5
 * Goal:
 *   test of xxx(...)
 ******************************************************************************/
void test5(){
  fprintf(stdout,"test5: NOT IMPLEMENTED\n");
  return;
}
/******************************************************************************
 *                       test6
 * Goal:
 *   test of xxx(...)
 ******************************************************************************/
void test6(){
  fprintf(stdout,"test6: NOT IMPLEMENTED\n");
  return;
}

/*############################################################################*/
/*#                      tests                                               #*/
/*############################################################################*/
void dfb_test(){
  test1(); test2(); test3(); // test4(); test5(); test6();
}

/* EOF */
