/*******************************************************************************
 * Author: Jerome Dubois       Jerome.Dubois@ensimag.imag.fr jerome@stud.ntnu.no
 * Copyright 2007 Jerome Dubois
 * 
 * 
 ******************************************************************************/
 
#ifndef _GRIM_H
#define _GRIM_H

#include <stdio.h> // for FILE

#define DEFINED
#undef  UNDEFINED
#define PI 3.14159265                // 3.141592653589793238462643383279
#define BLACK 0
#define WHITE 255
#define FALSE 0
#define TRUE  1

#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned char bit8;    // 8 bits // or uint8_t defined in stdint.h (C99)

/**
 * grim:
 * Type representing an image in 256 levels of grey
 * "grim" sould be read "Grey Image"
 */
typedef struct {
  int  w;     /* width  of image */
  int  h;     /* height of image */
  bit8 *pix;  /* array of pixels */
} grim;

typedef struct {
  int  w;     /* width  of image */
  int  h;     /* height of image */
  float *pix; /* array of pixels */
} fgrim;
/*
 * BMP:     _     GRIM:    0_1
 *      1 |c d|         0 |a b|
 *      0 |a_b|         1 |c_d|
 *         0 1          
 */
/*############################################################################*/
/*#                      functions                                           #*/
/*############################################################################*/

/* manipulation */
int  grim_create(grim *img, int width, int height);
int  grim_free(grim *img);
int  grim_copy(grim *in, grim *out);
int  grim_diff(grim *im1, grim *im2);
int  grim_cmp(grim *im1, grim *im2);
int  grim_scale(int fact, grim *in, grim *out);
int  grim_set_color(grim *img, bit8 color);
int fgrim_set_color(fgrim *fimg, float color);
int  grim_set_sinus(grim *img, bit8 color, float direction, int period);
int fgrim_set_sinus(fgrim *fimg, bit8 color, float direction, int period);
int  grim_resize(grim *in, grim *out, int new_w, int new_h);

int fgrim_create(fgrim *fimg, int width, int height);
int fgrim_free(fgrim *fimg);
int fgrim_copy(fgrim *in, fgrim *out);
float fgrim_diff(fgrim *fim1, fgrim *fim2);

int  grim_to_fgrim(grim *in, fgrim *fout);
int fgrim_to_grim(fgrim *fin, grim *out);

int  grim_line_to_2d(grim  *fin, int line, grim *out_2d);
int fgrim_line_to_2d(fgrim *fin, int line, grim *out_2d);

/* input/output */
int  grim_write(grim *img, FILE *fpout);
int fgrim_write(fgrim *fimg, FILE *fpout);
int  grim_print(grim *img);
int fgrim_print(fgrim *fimg);
int  grim_write_bmp(grim *img, char* bmp);
int fgrim_write_bmp(fgrim *fimg, char* bmp);
int  grim_read_bmp(char* bmp, grim *img);                      // NO CHECKS DONE
int  grim_write_grim(grim *imgin, char* fileout);
int  grim_read_grim(char* filein, grim *imgout);
int  grim_text_read(char* filein, grim *imgout);               // NO CHECKS DONE

#ifdef __cplusplus
}
#endif

#endif // #ifndef _GRIM_H

