/**
 * File   grim2bmp.c
 * Author Jérôme Dubois <Jerome.Dubois@ensimag.imag.fr>
 * Date   2007-10-24
 * Copyright 2007 Jerome Dubois
 *
 * Description
    create a BMP image from a GRIM image. See grim.h for more information.  
   ./grim2bmp [-o bmp_image] [-h] grim_image
     grim_image : input image to transform into BMP. Should have extention .grim
     bmp_image  : output image to write to. If it exists, it is overwritten.
                  default name is "out.bmp"
                  (should be input name but with .bmp extention.)
     -h : print help
 */
 /********************************************************************************/
#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit  */
#include <assert.h>    /* for assert */
#include <string.h> 
#include <getopt.h> 

#include "grim.h"

#define STRING_SIZE 256   // maximal size of strings, including '\0'

void printHelp(char * command);                        // print help message
void printError(char * error_message, char * command); // print error message

int main(int argc, char * argv[]){

    // Command line management 
    extern char *optarg;  // specific to getopt
    extern int optind;    //, opterr, optopt;  // id.
    char c;
    char arg_o[STRING_SIZE]="out.bmp";
    char arg_i[STRING_SIZE];
    int  option_o = 0 ;
 
    // Management of parameters in command line
    while ((c = getopt(argc, argv, "o:h")) != -1) {
        switch(c) {
        case 'h': printHelp(argv[0]); return EXIT_SUCCESS;
        case 'o':
            option_o = 1;
            assert(strlen(optarg) < STRING_SIZE); // check size
            strncpy(arg_o,optarg,STRING_SIZE);
            if (arg_o[0] == '-') printError("Bad Format for arg_i!",argv[0]);
            break;
        default: printError("Bad Format!",argv[0]);
        }
    }
    
    // parameter: arg_i
    if ((argc - optind) != 1) printError("Bad Format",argv[0]);
    assert(strlen(argv[optind]) < STRING_SIZE);
    strncpy(arg_i,argv[optind],STRING_SIZE);
    
    // parameter: arg_o
    if (!option_o) /* set default */;
    
    // verification
    //fprintf(stdout,"Valeur de arg_o: %s\n",arg_o);
    //fprintf(stdout,"Valeur de arg_i: %s\n",arg_i);
    
    // execution
    grim img;
    grim_read_grim(arg_i,&img);
    //fprintf(stdout,"%s: w=%d, h=%d\n",arg_i, img.w, img.h);
    grim_write_bmp(&img,arg_o);
    grim_free(&img);
    
    return EXIT_SUCCESS;
}


/** 
 * Print the help message  
 * @param command used (argv[0])
 */
void printHelp(char * command) {
    printf("NAME\n"                                                           );
    printf("      %s\n",command                                               );
    printf("\nSYNOPSIS\n"                                                     ); 
    printf("      %s [-h]\n",command                                          );
    printf("      %s [-o bmp_image] grim_image\n",command                     );
    printf("\nDESCRIPTION\n"                                                  ); 
    printf("      -h :print help and exit\n"                                  );
    printf("      -o bmp_image: a bmp file name\n"                            );
    printf("         Default value : \"out.bmp\"\n"                           );
    printf("\nAUTHOR\n"                                                       );
    printf("      Jerome Dubois <Jerome.Dubois@ensimag.imag.fr>\n"            );
    printf("\nCOPYRIGHT\n"                                                    );
    printf("      Copyright (c) Jérome Dubois\n"                              );
}

/**
 * print error message on stderr 
 * @param error_message Error message to print 
 * @param command       command used (argv[0])
 */
void printError(char * error_message, char * command) {
    fprintf(stderr, "[ERROR] %s\n",error_message);
    fprintf(stderr, "Use '%s -h' for help\n", command);
    exit(EXIT_FAILURE);
}
