/**
 * File   print_grim.c
 * Author Jérôme Dubois <Jerome.Dubois@ensimag.imag.fr>
 * Date   2007-10-24
 * Copyright 2007 Jerome Dubois
 *
 * Description
    Print on stdout a GRIM image as a human readable array of integers. The image
    should be small.
   ./print_grim [-h] grim_image
     grim_image  : image to print
     -h : print help
 */
 /********************************************************************************/
#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit  */
#include <assert.h>    /* for assert */
#include <string.h> 
#include <getopt.h> 

#include "grim.h"

#define STRING_SIZE 256   // maximal size of strings, including '\0'

void printHelp(char * command);                        // print help message
void printError(char * error_message, char * command); // print error message

int main(int argc, char * argv[]){

    // Command line management 
    extern char *optarg;  // specific to getopt
    extern int optind;    //, opterr, optopt;  // id.
    char c;
    char arg_i[STRING_SIZE];
 
    // Management of parameters in command line
    while ((c = getopt(argc, argv, "o:h")) != -1) {
        switch(c) {
        case 'h': printHelp(argv[0]); return EXIT_SUCCESS;
        default: printError("Bad Format!",argv[0]);
        }
    }
    
    // parameter: arg_i
    if ((argc - optind) != 1) printError("Bad Format",argv[0]);
    assert(strlen(argv[optind]) < STRING_SIZE);
    strncpy(arg_i,argv[optind],STRING_SIZE);
        
    // execution
    grim img;
    grim_read_grim(arg_i,&img);
    fprintf(stderr,"%s: w=%d, h=%d\n",arg_i, img.w, img.h);
    grim_print(&img);
    grim_free(&img);
    
    return EXIT_SUCCESS;
}


/** 
 * Print the help message  
 * @param command used (argv[0])
 */
void printHelp(char * command) {
    printf("NAME\n"                                                           );
    printf("      %s\n",command                                               );
    printf("\nSYNOPSIS\n"                                                     ); 
    printf("      %s [-h]\n",command                                          );
    printf("      %s grim_image\n",command                                     );
    printf("\nDESCRIPTION\n"                                                  ); 
    printf("      -h :print help and exit\n"                                  );
    printf("      Print on stdout a GRIM image as a human readable array of integers.\n"                            );
    printf("\nAUTHOR\n"                                                       );
    printf("      Jerome Dubois <Jerome.Dubois@ensimag.imag.fr>\n"            );
    printf("\nCOPYRIGHT\n"                                                    );
    printf("      Copyright (c) Jérome Dubois\n"                              );
}

/**
 * print error message on stderr 
 * @param error_message Error message to print 
 * @param command       command used (argv[0])
 */
void printError(char * error_message, char * command) {
    fprintf(stderr, "[ERROR] %s\n",error_message);
    fprintf(stderr, "Use '%s -h' for help\n", command);
    exit(EXIT_FAILURE);
}
