/*******************************************************************************
 * Author: Jérôme Dubois       Jerome.Dubois@ensimag.imag.fr jerome@stud.ntnu.no
 * Copyright 2007 Jerome Dubois
 * 
 * 
 ******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h> 
#include <getopt.h> 
#include <math.h>

#include "grim.h"

#define STRING_MAX_SIZE 128   // maximal size of strings (including '\0')
#define IMAGES "../images/"   // path to the images (to read and create)

void usage();
int test_1();
int twodir();
int test_3();
int test_4();
int test_5();
int test_6();

int main(int argc, char * argv[]){
  
  if (sizeof(char)!=1) fprintf(stderr,"Error: sizeof(char) = %d != 1\n",sizeof(char));
  if (test_1()!=EXIT_SUCCESS) fprintf(stderr,"Error: test_1 failed\n");
  if (twodir()!=EXIT_SUCCESS) fprintf(stderr,"Error: twodir failed\n");
  if (test_3()!=EXIT_SUCCESS) fprintf(stderr,"Error: test_3 failed\n");
  if (test_4()!=EXIT_SUCCESS) fprintf(stderr,"Error: test_4 failed\n");
  if (test_5()!=EXIT_SUCCESS) fprintf(stderr,"Error: test_5 failed\n");
  if (test_6()!=EXIT_SUCCESS) fprintf(stderr,"Error: test_6 failed\n");
  return EXIT_SUCCESS;
}

/*******************************************************************************
* usage
* 
*******************************************************************************/
void usage() {
    printf("usage: exemple [-p <param1>] <param2>\n");
    printf("\tbla as argument\n");
    printf("\t-p <param1> specify alternate bla (default xxx)\n");
}

/*******************************************************************************
* test_1
* 
*******************************************************************************/
int test_1(){
  int i, j, w, h;
  
  /* image white, 100x100 */
  grim white1, white2;
  grim_create(&white1, 100, 100);
  grim_set_color(&white1, WHITE);
  white2.w = white2.h = 100;
  white2.pix = malloc(100*100);
  memset((void*) white2.pix, 255, 100*100);
  if (! grim_cmp(&white1, &white2)) fprintf(stderr,"Error: Test: white1 != white2\n");
  grim_free(&white1);
  grim_free(&white2);

  /* image black 4x6 */
  grim black_4x6;
  grim_create(&black_4x6, 4, 6);
  for (i=0; i<4*6; i++) black_4x6.pix[i] = (bit8)0;
  grim black2;
  grim_create(&black2, 4, 6);
  grim_set_color(&black2, BLACK);
  if (! grim_cmp(&black_4x6, &black2)) fprintf(stderr,"Error: Test: black_4x6 != black2\n");
  grim_write_grim(&black_4x6, IMAGES"black_4x6.grim");
  grim_free(&black2);
  grim black3;
  grim_read_grim(IMAGES"black_4x6.grim", &black3);
  if (! grim_cmp(&black_4x6, &black3)) fprintf(stderr,"Error: Test: black_4x6 != black3\n");
  grim_free(&black_4x6);
  grim_free(&black3);

  /* image with multiple of 16. 4x4. */
  grim by16_4x4, by16bis;
  grim_create(&by16_4x4, 4, 4); /* do not free it */
  grim_create(&by16bis , 4, 4);
  by16_4x4.pix = (bit8*) "\0\20\40\60\100\120\140\160\200\220\240\260\300\320\340\360";
  for (i=0;i<16;i++) by16bis.pix[i]=16*i;
  if (! grim_cmp(&by16_4x4, &by16bis)) fprintf(stderr,"Error: Test: by16_4x4 != by16bis\n");
  grim_write_grim(&by16_4x4 ,  IMAGES"by16_4x4.grim");
  
  //grim by16_big;
  //grim_scale(20, &by16_4x4, &by16_big);
  //grim_write_grim(&by16_big, IMAGES"by16_big.grim");
  //grim_free(&by16_big);
  
  grim_free(&by16bis);
  
  /* image with 0,1,2,3,4,...,16 and scales image */
  grim suite_4x4;
  grim_create(&suite_4x4, 4, 4);
  for (i=0; i<4*4; i++) suite_4x4.pix[i] = (bit8)i;
  grim_write_grim(&suite_4x4, IMAGES"suite_4x4.grim");
  grim suite_4x4_fact3;
  grim_scale(3, &suite_4x4, &suite_4x4_fact3);
  grim_write_grim(&suite_4x4_fact3, IMAGES"suite_4x4_fact3.grim");
  grim_free(&suite_4x4_fact3);
  
  /* read GRIM file */
  grim fromfile;
  grim_read_grim(IMAGES"suite_4x4.grim", &fromfile);
  if (! grim_cmp(&suite_4x4, &fromfile)) fprintf(stderr,"Error: Test: (grim) suite_4x4 != fromfile\n");
  grim_free(&fromfile);
  
  /* write-read a BMP file */
  grim_write_bmp(&suite_4x4, IMAGES"suite_4x4.bmp");
  grim_read_bmp(IMAGES"suite_4x4.bmp", &fromfile);
  if (! grim_cmp(&suite_4x4, &fromfile)) fprintf(stderr,"Error: Test: (bmp ) suite_4x4 != fromfile\n");
  //grim_print(&suite_4x4); grim_print(&fromfile);
  grim_free(&suite_4x4);
  grim_free(&fromfile);
  
  /* write a BMP with sinus image */
  grim sinus1;
  grim_create(&sinus1, 200, 300);
  grim_set_sinus(&sinus1, 200, 11*PI/12.0, 100);
  grim_write_bmp(&sinus1, IMAGES"sinus1.bmp");
  grim_free(&sinus1);
  //fprintf(stdout,"Info: Image %ssinus1.bmp has been written. Please check that it is correct.\n", IMAGES);
  
  /* convert to fgrim and compare */
  i = 150; j = 250;
  grim sinus2, sinus3;
  fgrim fsinus2, fsinus3;

  grim_create(&sinus2, i, j);
  grim_set_sinus(&sinus2, WHITE, 20*PI/12.0, 60);
  grim_to_fgrim(&sinus2, &fsinus2);
  fgrim_copy(&fsinus2, &fsinus3);
  fgrim_to_grim(&fsinus3, &sinus3);
  i = grim_diff(&sinus2, &sinus3);
  j = 2; // threshold. eg: 5
  if (i<0 || i> j ) fprintf(stderr,"Error: Test: [fgrim] diff(&sinus2,&sinus3) is %d (and threshold is %d)\n",i,j);
  
  grim_free(&sinus2);
  grim_free(&sinus3);
  fgrim_free(&fsinus2);
  
  /* fgrim_set_sin */
  fgrim sinus4;
  grim  sinus4_grim, sinus4_grim_2;
  w=250; h=100;
  fgrim_create(&sinus4, w, h);
  fgrim_set_sinus(&sinus4, 100, 1*PI/3.0, 100);
  fgrim_to_grim(&sinus4, &sinus4_grim);
  grim_resize(&sinus4_grim, &sinus4_grim_2, 300, 200);
  fgrim_free(&sinus4);
  grim_write_bmp(&sinus4_grim_2, IMAGES"sinus4.bmp");
  grim_free(&sinus4_grim  );
  grim_free(&sinus4_grim_2);
  
  /* grim_resize */
  grim resize_in, resize_out;
  grim_read_bmp(IMAGES"passbilde_small.bmp", &resize_in);
  grim_resize(&resize_in, &resize_out, resize_in.w*2.5, resize_in.h*2);
  grim_write_bmp(&resize_out, IMAGES"passbilde_small_big.bmp");
  grim_free(&resize_in);
  grim_free(&resize_out);
  
  /* grim_free */
  grim_free(&resize_in); // freed for the second time
  //grim not_allocated_grim;
  //grim_free(&not_allocated_grim); // free a non allocated pointer.

  return EXIT_SUCCESS;
}

/*******************************************************************************
* twodir: creates an image (twodir.{bmp,grim}) which contains info in two
* directions.
* 
*******************************************************************************/
int twodir(){
  int i, j, w, h;
  double t;
  fgrim sin_1pi4, sin_3pi4, twodir;
  grim twodir_grim, grtmp, grtmp2;
  
  // computation of twodir
  w = 300;
  h = w; // the image should be squared for the dfb // ATTENTION
  fgrim_create(&sin_1pi4, w, h);
  fgrim_set_sinus(&sin_1pi4, WHITE, 1*PI/4.0, 60);
    fgrim_to_grim(&sin_1pi4, &grtmp);
    grim_resize(&grtmp, &grtmp2, w+20, h+20);
    grim_write_bmp(&grtmp2, IMAGES"sin_1pi4.bmp"); // &grtmp or &grtmp2
    grim_free(&grtmp);
    grim_free(&grtmp2);
  
  fgrim_create(&sin_3pi4, w, h);
  fgrim_set_sinus(&sin_3pi4, WHITE, 3*PI/4.0, 30);
    //printf("sin_3pi4:\n"); fgrim_print(&sin_3pi4);
    fgrim_to_grim(&sin_3pi4, &grtmp);
    grim_resize(&grtmp, &grtmp2, w+20, h+20);
    grim_write_bmp(&grtmp2, IMAGES"sin_3pi4.bmp"); // &grtmp2
    grim_free(&grtmp);
    grim_free(&grtmp2);
    
  fgrim_create(&twodir, w, h);
  for (i=0; i<h; i++){ // for each raw
    for (j=0; j<w; j++){ // for each column
      t = 1 * sin_1pi4.pix[i*w+j] + 1 * sin_3pi4.pix[i*w+j] ;
      twodir.pix[i*w+j] = t/2.0 ; // assert(value is in [0,255])
    }
  }
  fgrim_free(&sin_1pi4);
  fgrim_free(&sin_3pi4);
  
  // export of twodir
  fgrim_to_grim(&twodir,&twodir_grim);
  fgrim_free(&twodir);
  grim_write_grim(&twodir_grim, IMAGES"twodir.grim");
  grim_write_bmp(&twodir_grim, IMAGES"twodir.bmp");
  grim_free(&twodir_grim);
  grim_free(&grtmp);

  return EXIT_SUCCESS;
}

/*******************************************************************************
* test_3: test grim_set_sinus
* 
*******************************************************************************/
int test_3(){
int w, h;
  grim g1;
  
  w = 80;
  h = w;
  grim_create(&g1, w, h);
  grim_set_sinus(&g1, WHITE, 1*PI/4.0, 5);
  grim_write_bmp(&g1, IMAGES"test_3.bmp");
  grim_free(&g1);
  

  return EXIT_SUCCESS;
}

/*******************************************************************************
* test_4: draw an horizontal sinc
* 
*******************************************************************************/
int test_4(){ // sinc
  int i, j, w, h, pixelval;
  double step, sincofstep;
  fgrim fg;
  grim g;
  
  // computation of twodir
  w = 200;
  h = 25;

  fgrim_create(&fg, w, h);
  
  for (j=0; j<w; j++){ // for each column
      step = (j-w/2) /5.0 ;
      sincofstep = ( fabs(step) > 0 ) ? sin(step)/step : 1 ;
      pixelval = 256 - (sincofstep*128 +128);
      (pixelval > 255) ? (pixelval=255) : 0 ;
      //printf("j=%3d, step=% 3.5f, sincofstep=% f, pixelval=%d\n", j, step, sincofstep, pixelval);
      for (i=0; i<h; i++){ // for each raw
          fg.pix[i*w+j] = pixelval ;
      }
  }
  fgrim_to_grim(&fg, &g);
  grim_write_bmp(&g, IMAGES"horizontal_sinc.bmp");
  

  return EXIT_SUCCESS;
}

/*******************************************************************************
* test_5: test of grim_line_to_2d
* 
*******************************************************************************/
int test_5(){
  int w, h, i;
  float val;
  fgrim i0, i1, i2, i3, i4, i5;
  grim o0_1d, o0_2d;
  w = 500;
  h = 256;
  
  fgrim_create(&i1, w, 1); fgrim_set_sinus(&i1, WHITE, 0, 20);
  fgrim_create(&i2, w, 1); fgrim_set_sinus(&i2, WHITE, 0, 40);
  fgrim_create(&i3, w, 1); fgrim_set_sinus(&i3, WHITE, 0, 60);
  fgrim_create(&i4, w, 1); fgrim_set_sinus(&i4, WHITE, 0, 80);
  fgrim_create(&i5, w, 1); fgrim_set_sinus(&i5, WHITE, 0, 100);
  
  fgrim_create(&i0, w, 1);
  for (i=0; i<w; i++){
    val = 1 * i1.pix[i] + 1 * i2.pix[i] + 1 * i3.pix[i] + 1 * i4.pix[i] + 1 * i5.pix[i] ;
    i0.pix[i] = val / 5 ;
  }
  
  fgrim_to_grim(&i0, &o0_1d);
  grim_line_to_2d(&o0_1d, 0, &o0_2d);
  grim_write_bmp(&o0_2d, IMAGES"o0_2d.bmp");
  
  fgrim_free(&i0);
  fgrim_free(&i1);
  fgrim_free(&i2);
  fgrim_free(&i3);
  fgrim_free(&i4);
  fgrim_free(&i5);
  grim_free(&o0_1d);
  grim_free(&o0_2d);

  return EXIT_SUCCESS;
}

/*******************************************************************************
* test_6: 
* 
*******************************************************************************/
int test_6(){
  int w, h;
  fgrim f;
  grim g;
  w = 128;
  h = 128;
  
  fgrim_create( &f, w, h);
  fgrim_set_sinus( &f, WHITE, 0, 5);
  fgrim_to_grim( &f, &g);
  grim_write_grim( &g, IMAGES"sin_0_5_128x128.grim");
  grim_free( &g);
  fgrim_free( &f);
  
  fgrim_create( &f, w, h);
  fgrim_set_sinus( &f, WHITE, 0, 20);
  fgrim_to_grim( &f, &g);
  grim_write_grim( &g, IMAGES"sin_0_20_128x128.grim");
  grim_free( &g);
  fgrim_free( &f);
  
  fgrim_create( &f, w, h);
  fgrim_set_sinus( &f, WHITE, 0, 50);
  fgrim_to_grim( &f, &g);
  grim_write_grim( &g, IMAGES"sin_0_50_128x128.grim");
  grim_free( &g);
  fgrim_free( &f);

  return EXIT_SUCCESS;
}

// -EOF-
