
source('pair_filter.m');
source('rev_fpv.m');
source('reorganize_v2h.m');
source('rev_fph.m');
source('upsample.m');
source('PIshift.m');

function matB1 = rev_dfb( matB8b1,matB8b2, f)
% function rev_dfb (reverse directional filterbank)
% Return: the output of a reverse two band directional filterbank
% Param:
%    f : filter coef
%    matB8b1, matB8b1: the two subbands to useto recompose output image
%
    
    % param check
    n = size(matB8b1)(2);
    if ( 2*n != size(matB8b1)(1) ) disp('Error (rev_dfb): The input matrices should have 2w=h'); end
    if ( mod(n         ,2) != 0 ) disp('Error (rev_dfb): The input matrices must have even dimensions'); end
    if ( size(matB8b1) != size(matB8b2) ) disp('Error (rev_dfb): The input matrices must have the same dimensions'); end
    if ( mod(size(f)(2),2) != 0 ) disp('Error (rev_dfb): The filter must have even dimensions'); end
    
    % filter preparation
    f0 = f
    f1 = pair_filter(f)
    
    matB7b1 = ( matB8b1 + matB8b2 )/2
    matB7b2 = ( matB8b1 - matB8b2 )/2
    
    matB6b1 = rev_fpv( matB7b1, f ) ;
    matB6b2 = rev_fpv( matB7b2, f ) ;
    
    matB5b1 = reorganize_v2h( matB6b1 ) ;
    matB5b2 = reorganize_v2h( matB6b2 ) ;
    
    matB4b1 = rev_fph( matB5b1, f ) ;
    matB4b2 = rev_fph( matB5b2, f ) ;
    
    matB2 = upsample( matB4b1, matB4b2 ) ;
    
    matB1 = PIshift(matB2) ;
return

